/*
	main.c		MW CodeWarrior 11 C source file
	1997 Alastair Matthews
	
	A simple modal demonstration of how to use the functions in QuickChar.c
	to support the QuickChar control panel in your applications.
	
	We have a single dialog with a popup menu of available fonts,
	and a useritem that contains a sample of the selected font.
	When the one of the QuickChar pickers is used in this dialog,
	its characters appear in the selected font.
	Nothin happens if you select a character in the picker, since there's no field
	in which the character could be entered. However, you don't need to worry about this
	in your programs, because the character will be sent to you like any other
	keyDown/autoKey event for you to handle.
	
	This demonstrates the main methods of supporting QuickChar; the only
	technique not covered is suspend/resume and activate/deactivate because our app
	is modal.
	
	The pieces of code specific to QuickChar are flagged by 		QChar		
	so you can ignore the normal dialog handling stuff.
	Lots of other "proper" stuff like error checking has been left out
	as I wanted to make this demo as simple as possible.
	
	If there's demand I'll included more examples, but this one should get you started for now.
	Also note that the process is exactly the same in ppc native code, so I haven't wasted
	archive space with a separate project.
	
	Enjoy!
	
	Note: Feel free to use the xxxxUtils files for your own use, if you want.
*/

#include	"main.h"

#include	"dlogUtils.h"
#include	"cntlUtils.h"

#include	"QuickChar.h"		//		QChar	

//	The main program
void	main(void)
{
	initToolboxAndMemory();
	
	initQuickChar();	//		QChar	
					//	(note: above is our own routine, not a QuickChar one.
					//	QuickChar routines are prefix QChar
	//	Continue to run if uninstalled, but beep to say so
	//	All QC.c functions check for QC's presence so you needn't
	//	worry about that.
	if (!QCharInstalled())
		SysBeep(30);
		
	doDemoDialog();
	
	QCharSetAware(false);	//		QChar	
	
	ExitToShell();
}

//	Setup managers etc
void	initToolboxAndMemory(void)
{	short			i;
	
	MaxApplZone();

	for (i = 1; i <= 3; i++)
		MoreMasters();		//	alloc some master pointer blocks

	//	init managers
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
}

//	Initialise QuickChar
void	initQuickChar(void)		//		QChar	
{
	QCharInit();				//	setup internal variables
	QCharSetAware(true);			//	Let QuickChar know that we support it :)
	QCharSetFont(systemFont);		//	Initialise the current font
}

//	Display the demo dialog
void	doDemoDialog(void)
{
	DialogRef		d = nil;
	short			item = 0;
	OSErr		err = noErr;
	ControlRef		popupCntl;
	MenuRef		popupMenu;
	short			fontNum;
	
	d = GetNewDialog(kDemoDialogId, nil, (WindowRef)-1L);
	
	err = SetDialogDefaultItem(d, kStdOkItemIndex);
	
	//	Add fontnames to the popup
	popupCntl = getDlogControl(d, kPopupItem);
	popupMenu = getPopupsMenu(popupCntl);
	AppendResMenu(popupMenu, 'FONT');
	
	//	Initialise popup
	SetControlMinimum(popupCntl, 1);
	SetControlMaximum(popupCntl, CountMenuItems(popupMenu));
	SetControlValue(popupCntl, 1);
	
	//	Now install our useritem which samples the selected font:
	setDlogUserItemProc(d, kSampleUI, NewUserItemProc(mySampleProc));
	
	//	Get the current selected font (item 1 in menu)'s number:
	//	And tell QuickChar that it is now the active font:
	fontNum = getSelectedFontNumber(d);						//		QC	
	QCharSetFont(fontNum);
	
	//	Display dialog
	ShowWindow(d);
	
	while (item != kStdOkItemIndex)
	{
		ModalDialog(nil, &item);
		
		switch (item)
		{
			case kPopupItem:
				//	Force the dialog to update
				//	so that the useritem proc
				//	can draw in the new font
				DrawDialog(d);
				
				//	Get the font number of the selected font,
				//	and pass it to quickchar
				fontNum = getSelectedFontNumber(d);			//		QC	
				QCharSetFont(fontNum);
				break;
		}
	}
	
	//	We're done
	DisposeDialog(d);
	
	//	This isn't strictly neccessary here, since we are going to
	//	quit immediately anyway, but if you have an app where
	//	the window underneath the dialog with styled text
	//	doesn't respond to an activate event by resetting the active font
	//	or, more likely, say in a multi-document application,
	//	if there is no window underneath the dialog it's wise to
	//	reset the active font to the system font again:
	QCharSetFont(systemFont);								//		QC	
	
}

//	Calculate number of selected font
short		getSelectedFontNumber(DialogRef d)
{
	ControlRef		popupCntl;
	MenuRef		popupMenu;
	Str255		fontName;
	short			fontNum;
	
	popupCntl = getDlogControl(d, kPopupItem);
	popupMenu = getPopupsMenu(popupCntl);

	GetMenuItemText(popupMenu, GetControlValue(popupCntl), fontName);
	
	GetFNum(fontName, &fontNum);
	return fontNum;
}

//	Sample the font
pascal	void	mySampleProc(DialogRef d, short item)
{
	Str255		text;
	Rect			rect;
	short			saveFont;
	
	saveFont = d->txFont;
	TextFont(getSelectedFontNumber(d));
	
	GetIndString(text, kMyStringsListId, kSampleIndex);
	
	getDlogItemRect(d, item, &rect);
	
	TETextBox(&text[1], text[0], &rect, teJustLeft);
	
	TextFont(saveFont);
}